
// Label Chooser for Magic Launch
// Copyright 2010 Michel Fortin
//
// This file is published under the terms of the Boost Software License 1.0.
//
// Boost Software License - Version 1.0 - August 17th, 2003
//
// Permission is hereby granted, free of charge, to any person or organization
// obtaining a copy of the software and accompanying documentation covered by
// this license (the "Software") to use, reproduce, display, distribute,
// execute, and transmit the Software, and to prepare derivative works of the
// Software, and to permit third-parties to whom the Software is furnished to
// do so, all subject to the following:
//
// The copyright notices in the Software and this entire statement, including
// the above license grant, this restriction and the following disclaimer,
// must be included in all copies of the Software, in whole or in part, and
// all derivative works of the Software, unless such copies or derivative
// works are solely in the form of machine-executable object code generated by
// a source language processor.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
// SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
// FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
// DEALINGS IN THE SOFTWARE.

#import "MLLabelChooser.h"

#ifdef ML_MAGIC_LAUNCH
// Magic Launch uses its own macro for localized strings. When compiling
// Magic Launch, replace NSLocalizedString with MLLocalizedString.
#import "MLBundle.h"
#undef NSLocalizedString
#define NSLocalizedString MLLocalizedString
#endif

// Layout spacing values
#define MLLabelButtonSize 18
#define MLLabelButtonMargin 4
#define MLLabelIconMargin 5

// Special ordering to mimick Finder's label order
#define MLFileLabelCount 8
static const short MLFileLabelOrder[MLFileLabelCount] = { 0, 6, 7, 5, 2, 4, 3, 1 };

// Function to create the icon for a label given the label color.
static NSImage *MLLabelIcon(NSColor *labelColor);

// Button cell for proper label button drawing and reacting to the mouse.
@interface MLLabelButtonCell : NSButtonCell {
	BOOL mouseInside;
	MLLabelChooser *labelChooser;
}

@property (assign) MLLabelChooser *labelChooser;

@end

@implementation MLLabelChooser

@synthesize labelText;

- (id)initWithFrame:(NSRect)frame {
    self = [super initWithFrame:frame];
    if (self) {
		NSMutableArray *labelButtonsMutable = [[NSMutableArray alloc] initWithCapacity:8];
		labelButtons = labelButtonsMutable;
		
		NSRect currentFrame = NSMakeRect(0, 0, MLLabelButtonSize, MLLabelButtonSize);
        NSArray *colors = [[NSWorkspace sharedWorkspace] fileLabelColors];
		
		size_t labelCount = [colors count];
		BOOL useCustomOrder = labelCount == MLFileLabelCount;
		for (size_t index = 0; index < labelCount; ++index) {
			NSButton *colorButton = [[NSButton alloc] initWithFrame:currentFrame];
			size_t orderedIndex = (useCustomOrder ? MLFileLabelOrder[index] : index);
			{
				MLLabelButtonCell *cell = [[MLLabelButtonCell alloc] init];
				if (index != 0)
					[cell setImage:MLLabelIcon([colors objectAtIndex:orderedIndex])];
				[cell setShowsBorderOnlyWhileMouseInside:YES];
				[colorButton setCell:cell];
				[cell release];
				cell.labelChooser = self;
			}
			[colorButton setAction:@selector(updateLabelValue:)];
			[colorButton setTarget:self];
			[colorButton setTag:orderedIndex];
			
			[self addSubview:colorButton];
			[labelButtonsMutable addObject:colorButton];
			if (index == 0)
				[colorButton setState:NSOnState];
			
			// Shift rect for next button
			if (index == 0)
				currentFrame.origin.x += MLLabelButtonMargin+1;
			currentFrame.origin.x += MLLabelButtonSize;
			[colorButton release];
		}
    }
    return self;
}

- (void)dealloc {
	[labelButtons release];
	[labelText release];
	[super dealloc];
}

- (void)sizeToFit {
	NSRect frame = [self frame];
	NSArray *colors = [[NSWorkspace sharedWorkspace] fileLabelColors];
	frame.size.width = [colors count]*MLLabelButtonSize + MLLabelButtonMargin+1;
	frame.size.height = MLLabelButtonSize;
	[self setFrame:frame];
}

- (int)intValue {
	return label;
}

- (void)setIntValue:(int)anInt {
	if (anInt != label) {
		[self willChangeValueForKey:@"intValue"];
		label = anInt;
		[self didChangeValueForKey:@"intValue"];
	}
	for (NSButton *button in labelButtons) {
		[button setState:([button tag] == label ? NSOnState : NSOffState)];
	}
}

- (void)updateLabelValue:(id)sender {
	[self setIntValue:[sender tag]];
	[[self target] performSelector:[self action] withObject:self];
}

- (void)updateMouseover:(id)sender {
	if (labelText) {
		int labelTag = [sender tag];
		NSString *labelString;
		if (labelTag == 0)
			labelString = @"";
		else {
			NSString *labelName = [[[NSWorkspace sharedWorkspace] fileLabels] objectAtIndex:labelTag];
			labelString = [NSString stringWithFormat:NSLocalizedString(@"“%@”", "Quoted label name for the label chooser"), labelName];
		}
		[labelText setStringValue:labelString];
	}
}

@end



@implementation MLLabelButtonCell

@synthesize labelChooser;

- (void)mouseEntered:(NSEvent *)event {
	mouseInside = YES;
	NSView *control = [self controlView];
	[control setNeedsDisplay:YES];
	[labelChooser updateMouseover:control];
}

- (void)mouseExited:(NSEvent *)event {
	mouseInside = NO;
	NSView *control = [self controlView];
	[control setNeedsDisplay:YES];
	[labelChooser updateMouseover:nil];
}

- (void)drawWithFrame:(NSRect)cellFrame inView:(NSView *)controlView {
	int state = [self state];
	if (state == NSOnState || mouseInside) {
		NSRect borderFrame = cellFrame;
		borderFrame.origin.x += 1;
		borderFrame.origin.y += 1;
		borderFrame.size.width -= 2;
		borderFrame.size.height -= 2;
		if (state == NSOnState || [self mouseDownFlags])
			[[NSColor whiteColor] set];
		else
			[[NSColor colorWithCalibratedWhite:0.5 alpha:0.25] set];
		[[NSBezierPath bezierPathWithRect:borderFrame] fill];
		[[NSColor colorWithCalibratedWhite:0.0 alpha:0.33] set];
		[[NSBezierPath bezierPathWithRect:borderFrame] stroke];
	}
	
	NSImage *image = [self image];
	if (image) {
		NSShadow *theShadow = [[NSShadow alloc] init];
		[theShadow setShadowOffset:NSMakeSize(0.0, -1.0)];
		[theShadow setShadowBlurRadius:1.5];
		[theShadow setShadowColor:[NSColor colorWithCalibratedWhite:0.0 alpha:0.70]];
		[theShadow set];
		
		[image drawInRect:cellFrame fromRect:cellFrame operation:NSCompositeSourceOver fraction:1.0];
		
		[theShadow release];
	} else {
		NSRect xRect = NSMakeRect(MLLabelIconMargin+0.5, MLLabelIconMargin+0.5, 
								  MLLabelButtonSize-2*(MLLabelIconMargin+0.5), MLLabelButtonSize-2*(MLLabelIconMargin+0.5));
		NSBezierPath *xPath = [NSBezierPath bezierPath];
		[xPath moveToPoint:xRect.origin];
		[xPath lineToPoint:NSMakePoint(xRect.origin.x + xRect.size.width, xRect.origin.y + xRect.size.height)];
		[xPath moveToPoint:NSMakePoint(xRect.origin.x, xRect.origin.y + xRect.size.height)];
		[xPath lineToPoint:NSMakePoint(xRect.origin.x + xRect.size.width, xRect.origin.y)];
		[xPath setLineWidth:1.5];
		
		[[NSColor colorWithCalibratedWhite:0.2 alpha:1.0] set];
		[xPath stroke];
	}
}

@end

NSImage *MLLabelIcon(NSColor *labelColor) {
	NSImage *icon = [[NSImage alloc] initWithSize:NSMakeSize(MLLabelButtonSize, MLLabelButtonSize)];
	[icon lockFocus];
	[NSGraphicsContext saveGraphicsState];
	
	NSBezierPath *clipShape = [NSBezierPath bezierPath];
	[clipShape appendBezierPathWithRect:NSMakeRect(MLLabelIconMargin, MLLabelIconMargin-1, 
												   MLLabelButtonSize-2*MLLabelIconMargin, MLLabelButtonSize-2*MLLabelIconMargin)];
	
	NSGradient *theGradient = [[NSGradient alloc] initWithColorsAndLocations:
							   labelColor, (CGFloat)0.0,
							   [NSColor whiteColor], (CGFloat)2.0,
							   nil];
	
	[theGradient drawInBezierPath:clipShape angle:-90];
	
	[NSGraphicsContext restoreGraphicsState];
	[theGradient release];
	
	[icon unlockFocus];
	return icon;
}

